<?php

namespace Drupal\email_content_templates_commerce\Plugin\EctMail;

use Drupal\commerce\ConditionGroup;
use Drupal\commerce\Plugin\Commerce\Condition\ParentEntityAwareInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\email_content_templates\Annotation\EctMailPlugin;
use Drupal\email_content_templates\Plugin\EctMail\EctMailPluginBase;
use Drupal\email_content_templates\Entity\EmailContentTemplateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the email_content_templates_info.
 *
 * @EctMailPlugin(
 *   id = "commerce-order_receipt",
 *   label = @Translation("Commerce order receipt"),
 *   category = "Commerce",
 * )
 */
class CommerceOrderReceipt extends EctMailPluginBase implements EctCommerceOrderMailPluginInterface {

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
    );

    $instance->conditionsManager = $container->get('plugin.manager.commerce_condition');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function paramTokenMap() {
    return [
      'order' => 'commerce_order',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConditionsForm(EmailContentTemplateInterface $template, FormStateInterface $form_state) {
    $form['conditions'] = [
      '#type' => 'commerce_conditions',
      '#title' => $this->t('Conditions'),
      '#parent_entity_type' => 'commerce_payment_gateway',
      '#entity_types' => ['commerce_order'],
      '#default_value' => $template->getConditions()['conditions'] ?? [],
      '#parents' => ['conditions'],
    ];
    $form['conditionOperator'] = [
      '#type' => 'radios',
      '#title' => $this->t('Condition operator'),
      '#title_display' => 'invisible',
      '#options' => [
        'AND' => $this->t('All conditions must pass'),
        'OR' => $this->t('Only one condition must pass'),
      ],
      '#default_value' => $template->getConditions()['conditionOperator'] ?? 'AND',
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function preRenderAlterMail(EmailContentTemplateInterface $template, array &$message): void {
    $order = $message['params']['order'];

    // Render order entity as body.
    // $build = \Drupal::entityTypeManager()->getViewBuilder($order->getEntityTypeId())->view($order, 'email');
    // @todo consider providing a setting to choose.
    // wether to render entity or theme or if even necessary.
    $body = [
      '#theme' => 'commerce_order_receipt',
      '#order_entity' => $order,
      '#totals' => \Drupal::service('commerce_order.order_total_summary')->buildTotals($order),
    ];
    if ($billing_profile = $order->getBillingProfile()) {
      $profile_view_builder = \Drupal::entityTypeManager()->getViewBuilder('profile');
      $body['#billing_information'] = $profile_view_builder->view($billing_profile);
    }

    $message['params']['body'] = $body;
    $message['params']['order_number'] = $message['params']['order']->getOrderNumber();
    $message['params']['store'] = $message['params']['order']->getStore()->getName();
  }

  /**
   * {@inheritdoc}
   */
  public function applies(EmailContentTemplateInterface $template, array $message): bool {
    $plugin = $template->getEmailPlugin();
    if (empty($plugin)) {
      return FALSE;
    }

    $condition_settings = $template->getConditions();
    if (!isset($condition_settings['conditions']) || count($condition_settings['conditions']) === 0) {
      // Templates without conditions always apply.
      return TRUE;
    }

    $conditions = $this->loadConditions($template);
    $conditions = array_filter($conditions, function ($condition) {
      /** @var \Drupal\commerce\Plugin\Commerce\Condition\ConditionInterface $condition */
      return $condition->getEntityTypeId() == 'commerce_order';
    });
    $condition_group = new ConditionGroup($conditions, $condition_settings['conditionOperator']);

    return $condition_group->evaluate($message['params']['order']);
  }

  /**
   * Gets the conditions.
   *
   * @return \Drupal\commerce\Plugin\Commerce\Condition\ConditionInterface[]
   *   The conditions.
   */
  protected function loadConditions(EmailContentTemplateInterface $template): array {
    $conditions = [];
    foreach ($template->getConditions()['conditions'] as $condition_settings) {
      $condition = $this->conditionsManager->createInstance($condition_settings['plugin'], $condition_settings['configuration']);
      if ($condition instanceof ParentEntityAwareInterface) {
        $condition->setParentEntity($this);
      }
      $conditions[] = $condition;
    }
    return $conditions;
  }

}
