<?php

namespace Drupal\email_content_templates\Form;

use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\email_content_templates\Entity\EmailContentTemplateTypeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form controller for the email content template entity edit forms.
 */
class EmailContentTemplateForm extends ContentEntityForm {

  /**
   * The email info plugin manager.
   *
   * @var \Drupal\email_content_templates\EctMailPluginManager
   */
  protected $emailInfoManager;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The condition plugin manager.
   *
   * @var \Drupal\Core\Condition\ConditionManager
   */
  protected $conditionsManager;

  /**
   * The context repository service.
   *
   * @var \Drupal\Core\Plugin\Context\ContextRepositoryInterface
   */
  protected $contextRepository;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = new static(
      $container->get('entity.repository'),
      $container->get('entity_type.bundle.info'),
      $container->get('datetime.time'),
    );

    $instance->emailInfoManager = $container->get('plugin.manager.ect_mail');
    $instance->dateFormatter = $container->get('date.formatter');
    $instance->conditionsManager = $container->get('plugin.manager.condition');
    $instance->contextRepository = $container->get('context.repository');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $entity = $this->getEntity();
    $result = $entity->save();

    $message_arguments = ['%label' => $this->entity->label()];
    $logger_arguments = $message_arguments;

    if ($result == SAVED_NEW) {
      $this->messenger()
        ->addStatus($this->t('New email content template %label has been created.', $message_arguments));
      $this->logger('email_content_templates')
        ->notice('Created new email content template %label', $logger_arguments);
    }
    else {
      $this->messenger()
        ->addStatus($this->t('The email content template %label has been updated.', $message_arguments));
      $this->logger('email_content_templates')
        ->notice('Updated new email content template %label.', $logger_arguments);
    }

    $form_state->setRedirect('entity.email_content_template.collection');
  }

  /**
   * {@inheritdoc}
   */
  public function getEntityFromRouteMatch(RouteMatchInterface $route_match, $entity_type_id) {
    /** @var \Drupal\email_content_templates\Entity\EmailContentTemplateInterface $entity */
    $entity = parent::getEntityFromRouteMatch($route_match, $entity_type_id);

    $bundle = $entity->getBundleEntity();
    if ($bundle instanceof EmailContentTemplateTypeInterface) {
      // Set default values if empty.
      if ($entity->get('subject')->isEmpty()) {
        $entity->setSubject($bundle->getDefaultSubject());
      }
      if ($entity->get('body')->isEmpty()) {
        $entity->set('body', [
          'value' => $bundle->getDefaultBody(),
          'format' => $bundle->getDefaultBodyFormat(),
        ]);
      }
    }

    return $entity;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form_state->setTemporaryValue('gathered_contexts', $this->contextRepository->getAvailableContexts());

    $form = parent::buildForm($form, $form_state);
    /** @var \Drupal\email_content_templates\Entity\EmailContentTemplateInterface $template */
    $template = $this->entity;

    // Add ajax handler on email plugin field.
    $form['email_plugin']['widget']['#ajax'] = [
      'callback' => '::buildConditions',
      'event' => 'change',
      'wrapper' => 'conditions-wrapper',
    ];
    $form['conditions'] = [
      '#prefix' => '<div id="conditions-wrapper">',
      '#suffix' => '</div>',
      '#weight' => 50,
      '#tree' => TRUE,
    ];

    $this->addConditions($form, $form_state, $this->entity);
    $form['meta'] = [
      '#type' => 'details',
      '#group' => 'advanced',
      '#weight' => -10,
      '#title' => $this->t('Status'),
      '#attributes' => ['class' => ['entity-meta__header']],
      '#tree' => TRUE,
    ];
    $form['meta']['published'] = [
      '#weight' => -120,
      '#type' => 'item',
      '#markup' => $template->isPublished() ? $this->t('Published') : $this->t('Not published'),
      '#access' => !$template->isNew(),
      '#wrapper_attributes' => ['class' => ['entity-meta__title']],
    ];
    $form['meta']['changed'] = [
      '#type' => 'item',
      '#title' => $this->t('Last saved'),
      '#weight' => -110,
      '#markup' => !$template->isNew() ? $this->dateFormatter->format($template->getChangedTime(), 'short') : $this->t('Not saved yet'),
      '#wrapper_attributes' => ['class' => ['entity-meta__last-saved']],
    ];
    $form['meta']['author'] = [
      '#type' => 'item',
      '#weight' => -100,
      '#title' => $this->t('Author'),
      '#markup' => $template->getOwner()->getAccountName(),
      '#wrapper_attributes' => ['class' => ['entity-meta__author']],
    ];
    if (isset($form['uid'])) {
      $form['author'] = [
        '#type' => 'details',
        '#title' => $this->t('Authoring information'),
        '#group' => 'advanced',
        '#weight' => 90,
        '#optional' => TRUE,
      ];
      $form['uid']['#group'] = 'author';
      if (isset($form['created'])) {
        $form['created']['#group'] = 'author';
      }
    }
    if ($this->moduleHandler->moduleExists('token')) {
      $email_plugin = $this->entity->getEmailPluginId();
      if (!empty($email_plugin) && $email_plugin !== '_none') {
        /** @var \Drupal\email_content_templates\Plugin\EctMail\EctMailPluginInterface $plugin */
        $plugin = \Drupal::service('plugin.manager.ect_mail')->createInstance($email_plugin);
        $token_map = $plugin->paramTokenMap();
        if (!empty($token_map)) {
          $form['body']['token_link'] = [
            '#theme' => 'token_tree_link',
            '#token_types' => array_values($token_map),
            '#global_types' => TRUE,
          ];
        }
      }
    }
    return $form;
  }

  /**
   * Ajax callback for conditions container.
   */
  public function buildConditions(array $form, FormStateInterface $form_state) {
    return $form['conditions'];
  }

  protected function addConditions(&$form, FormStateInterface $form_state, $template) {
    $input = $form_state->getUserInput();
    if (!empty($input) && $input['email_plugin'] !== '_none') {
      /** @var \Drupal\email_content_templates\Plugin\EctMail\EctMailPluginInterface $plugin */
      $plugin = \Drupal::service('plugin.manager.ect_mail')->createInstance($input['email_plugin']);
    }
    else {
      $plugin = $template->getEmailPlugin();
    }

    if (!empty($plugin)) {
      $conditions = $plugin->buildConditionsForm($template, $form_state);
      $form['conditions'] += $conditions;
    }
  }

}
