<?php

namespace Drupal\email_content_templates\Form;

use Drupal\Core\Entity\BundleEntityFormBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\email_content_templates\EctMailPluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form handler for email content template type forms.
 */
class EmailContentTemplateTypeForm extends BundleEntityFormBase {

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.ect_mail'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(EctMailPluginManager $email_info_manager) {
    $this->emailInfoManager = $email_info_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    /** @var \Drupal\email_content_templates\Entity\EmailContentTemplateTypeInterface $entity_type */
    $entity_type = $this->entity;
    if ($this->operation == 'add') {
      $form['#title'] = $this->t('Add email content template type');
    }
    else {
      $form['#title'] = $this->t(
        'Edit %label template type',
        ['%label' => $entity_type->label()]
      );
    }

    $form['label'] = [
      '#title' => $this->t('Label'),
      '#type' => 'textfield',
      '#default_value' => $entity_type->label(),
      '#description' => $this->t('The human-readable name of this email content template type.'),
      '#required' => TRUE,
      '#size' => 30,
    ];

    $form['description'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Description'),
      '#default_value' => $entity_type->getDescription(),
      '#description' => $this->t("Template type description."),
    ];

    $definitions = $this->emailInfoManager->getDefinitions();
    foreach ($definitions as $definition) {
      /*
       * D10.2 patch used.
       * @see https://www.drupal.org/project/drupal/issues/2269823
       */
      if (empty($definition['category'])) {
        $options[$definition['id']] = $definition['label'];
      }
      else {
        $options[$definition['category']][$definition['id']] = $definition['label'];
      }
    }
    asort($options);
    $form['email_plugins_wrapper'] = [
      '#type' => 'fieldset',
    ];
    $form['email_plugins_wrapper']['email_plugins'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Email plugins'),
      '#description' => $this->t('Select which the allowed mail plugins.'),
      '#options' => $options,
      '#default_value' => $entity_type->getEmailPluginIds(),
      '#required' => TRUE,
    ];

    $form['default_subject'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Default Subject'),
      '#maxlength' => 255,
      '#default_value' => $entity_type->getDefaultSubject(),
      '#description' => $this->t("Default subject."),
      '#required' => FALSE,
    ];

    $form['default_body'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Default Body'),
      '#description' => $this->t("Default Body."),
      '#required' => FALSE,
      '#default_value' => $entity_type->getDefaultBody(),
      '#format' => $entity_type->getDefaultBodyFormat(),
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $entity_type->id(),
      '#maxlength' => EntityTypeInterface::BUNDLE_MAX_LENGTH,
      '#machine_name' => [
        'exists' => [
          'Drupal\email_content_templates\Entity\EmailContentTemplateType',
          'load',
        ],
        'source' => ['label'],
      ],
      '#description' => $this->t('A unique machine-readable name for this email content template type. It must only contain lowercase letters, numbers, and underscores.'),
    ];

    return $this->protectBundleIdElement($form);
  }

  /**
   * {@inheritdoc}
   */
  protected function actions(array $form, FormStateInterface $form_state) {
    $actions = parent::actions($form, $form_state);
    $actions['submit']['#value'] = $this->t('Save template type');
    $actions['delete']['#value'] = $this->t('Delete template type');
    return $actions;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\email_content_templates\Entity\EmailContentTemplateTypeInterface $entity_type */
    $entity_type = $this->entity;

    $entity_type->setDefaultBody(trim($form_state->getValue('default_body')['value'] ?? ''));
    $entity_type->setDefaultBodyFormat(trim($form_state->getValue('default_body')['format'] ?? ''));
    $allowed_plugins = array_filter($form_state->getValue('email_plugins'));
    sort($allowed_plugins);
    $entity_type->setEmailPlugins($allowed_plugins);

    $status = $entity_type->save();
    $t_args = ['%name' => $entity_type->label()];
    if ($status == SAVED_UPDATED) {
      $message = $this->t('The template type %name has been updated.', $t_args);
    }
    elseif ($status == SAVED_NEW) {
      $message = $this->t('The template type %name has been added.', $t_args);
    }
    $this->messenger()->addStatus($message);
    $form_state->setRedirectUrl($entity_type->toUrl('collection'));
    return $status;
  }

}
