<?php

namespace Drupal\email_content_templates\Plugin\EctMail;

use Drupal\Component\Plugin\Exception\ContextException;
use Drupal\Component\Plugin\Exception\MissingValueContextException;
use Drupal\Core\Condition\ConditionAccessResolverTrait;
use Drupal\Core\Condition\ConditionPluginCollection;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContextAwarePluginInterface;
use Drupal\email_content_templates\Entity\EmailContentTemplateInterface;

/**
 * Abstract base class for template infos regarding the user module.
 */
abstract class EctUserMailMailPluginBase extends EctMailPluginBase {

  use ConditionAccessResolverTrait;

  /**
   * {@inheritdoc}
   */
  public function paramTokenMap() {
    return [
      'account' => 'user',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConditionsForm(EmailContentTemplateInterface $template, FormStateInterface $form_state) {
    // Adding conditions here.
    // User role and page path seem to be
    // the only meaningful conditions at this point, if any.
    $form['conditions'] = [
      '#type' => 'vertical_tabs',
      '#title' => $this->t('Conditions'),
      '#parents' => ['conditions'],
    ];

    $conditions = $template->getConditions();
    $definitions = $this->conditionsManager->getFilteredDefinitions('email_content_template', $form_state->getTemporaryValue('gathered_contexts'), ['email_content_template' => $template]);
    foreach ($definitions as $condition_id => $definition) {
      // Don't display the current theme condition.
      if ($condition_id == 'current_theme') {
        continue;
      }
      if ($condition_id == 'response_status') {
        // Hide response status condition.
        continue;
      }
      if (str_contains($condition_id, 'entity_bundle:')) {
        // Hide entity bundle related conditions (node/taxonomy).
        continue;
      }
      // Don't display the language condition until we have multiple languages.
      if ($condition_id == 'language' && !$this->language->isMultilingual()) {
        continue;
      }

      /** @var \Drupal\Core\Condition\ConditionInterface $condition */
      $condition = $this->conditionsManager->createInstance($condition_id, $conditions[$condition_id] ?? []);
      $form_state->set(['conditions', $condition_id], $condition);
      $condition_form = $condition->buildConfigurationForm([], $form_state);
      $condition_form['#type'] = 'details';
      $condition_form['#title'] = $condition->getPluginDefinition()['label'];
      $condition_form['#group'] = 'conditions';
      $form[$condition_id] = $condition_form;
    }

    if (isset($form['user_role'])) {
      $form['user_role']['#title'] = $this->t('Roles');
      unset($form['user_role']['roles']['#description']);
    }

    if (isset($form['request_path'])) {
      $form['request_path']['#title'] = $this->t('Pages');
      $form['request_path']['negate']['#type'] = 'radios';
      $form['request_path']['negate']['#default_value'] = (int) $form['request_path']['negate']['#default_value'];
      $form['request_path']['negate']['#title_display'] = 'invisible';
      $form['request_path']['negate']['#options'] = [
        $this->t('Show for the listed pages'),
        $this->t('Hide for the listed pages'),
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function applies(EmailContentTemplateInterface $template, array $message): bool {
    $plugin = $template->getEmailPlugin();
    if (empty($plugin)) {
      return FALSE;
    }

    $condition_settings = $template->getConditions();
    if (isset($condition_settings['conditions__active_tab'])) {
      unset($condition_settings['conditions__active_tab']);
    }
    if (count($condition_settings) === 0) {
      // Templates without conditions always apply.
      return TRUE;
    }

    foreach ($condition_settings as $condition_id => $settings) {
      // Required by DefaultLazyPluginCollection.
      $condition_settings[$condition_id]['id'] = $condition_id;
    }

    $conditions = [];
    $conditions_collection = new ConditionPluginCollection($this->conditionsManager, $condition_settings);
    foreach ($conditions_collection as $condition_id => $condition) {
      if ($condition instanceof ContextAwarePluginInterface) {
        try {
          $contexts = $this->contextRepository->getRuntimeContexts(array_values($condition->getContextMapping()));
          $this->contextHandler->applyContextMapping($condition, $contexts);
        }
        catch (MissingValueContextException $e) {
          return FALSE;
        }
        catch (ContextException $e) {
          return FALSE;
        }
      }
      $conditions[$condition_id] = $condition;
    }

    return $this->resolveConditions($conditions, 'and');
  }

}
